{-|
Module      : OpenAlex
Description : Textmining Collaborative Platform
Copyright   : (c) CNRS, 2023-Present
License     : AGPL + CECILL v3
Maintainer  : team@gargantext.org
Stability   : experimental
Portability : POSIX

@Gargantext@: search, map, share
-}

module OpenAlex
  ( module OpenAlex.Client
  , module OpenAlex.Types
  -- , fetchConcepts'
  , fetchConcepts
  , fetchWorks
  , fetchWorksC
  )
where

import Conduit
import qualified Data.Text as T
import Network.HTTP.Client (newManager, requestHeaders)
import Network.HTTP.Client.TLS (tlsManagerSettings)
import Protolude hiding (yield)
import OpenAlex.Client
import OpenAlex.ServantClientLogging
import OpenAlex.Types (ListOf(..), Meta(..), Page, PerPage, Cursor, Filter, Concept, Work)
import Servant.Client (BaseUrl(..), ClientEnv(..), ClientError, Scheme(Https), defaultMakeClientRequest, mkClientEnv, runClientM)

defaultClientEnv :: IO ClientEnv
defaultClientEnv = do
  manager <- newManager tlsManagerSettings
  let env' = mkClientEnv manager $ BaseUrl Https (T.unpack apiUrl) 443 ""
  let makeClientRequest bu req =
        let r = defaultMakeClientRequest bu req in
        r { requestHeaders = (requestHeaders r) <> [("User-Agent", "servant.client")]}
  let env = env' { makeClientRequest }
  pure $ addLoggingToClientEnv env


fetchConcepts :: Maybe Page
              -> Maybe PerPage
              -> Maybe Cursor
              -> Maybe Filter
              -> IO (Either ClientError (ListOf Concept))
fetchConcepts mPage mPerPage mCursor mFilter = do
  env <- defaultClientEnv
  runClientM (concepts mPage mPerPage mCursor mFilter) env

fetchWorks :: Maybe Page
           -> Maybe PerPage
           -> Maybe Cursor
           -> Maybe Filter
           -> IO (Either ClientError (ListOf Work))
fetchWorks mPage mPerPage mCursor mFilter = do
  env <- defaultClientEnv
  runClientM (works mPage mPerPage mCursor mFilter) env

fetchWorksC :: Maybe Cursor
            -> Maybe Filter
            -> IO (Either ClientError (Maybe Integer, ConduitT () Work IO ()))
fetchWorksC Nothing mFilter = do
  fetchWorksC (Just "*") mFilter
fetchWorksC mCursor mFilter = do
  env <- defaultClientEnv
  -- NOTE: per-page max is 200
  eRes <- runClientM (works (Just 1) (Just 1) Nothing mFilter) env
  case eRes of
    Left err -> pure $ Left err
    Right ListOf { meta = Meta { count }} -> do
      pure $ Right ( Just $ fromIntegral count
                   , producer env mCursor )
  where
    producer :: ClientEnv -> Maybe Cursor -> ConduitT () Work IO ()
    producer env mCursor' = do
      eRes <- liftIO $ runClientM (works Nothing (Just 200) mCursor' mFilter) env
      -- liftIO $ putText $ "Conduit fetching page with cursor " <> show mCursor'
      case eRes of
        Left err -> panic $ "error: " <> show err
        Right (ListOf { results, meta = _meta@(Meta { next_cursor }) }) -> do
          -- liftIO $ putText $ "Meta: " <> show meta
          yieldMany results
          producer env next_cursor
